<?php
/**
 * @copyright Copyright 2005-2010 RedIRIS, http://www.rediris.es/
 *
 * This file is part of phpPoA2.
 *
 * phpPoA2 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * phpPoA2 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with phpPoA2. If not, see <http://www.gnu.org/licenses/>.
 *
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License
 * @version 2.0
 * @author Jaime Perez <jaime.perez@rediris.es>
 * @filesource
 */

/**
 * This hook is executed right after retrieving the current URI, the params (both GET and POST) and
 * the arrays of allowed and denied patterns that will be checked inmediately.
 * It can be used to alter parameters and the URL, and also to configure the filters on runtime.
 * The hook receives the URI string, an array of parameters, the allowed and the denied patterns.
 * Functions for this hook must be defined like this:
 *
 * function queryBeforeFilterHook(&$uri, &$params, &$allowed, &$denied);
 *
 * Please bear in mind that hooks must return TRUE or they'll keep other hooks from executing.
 */
define("QUERY_BEFORE_FILTERS", "QUERY_BEFORE_FILTERS");

/**
 * Authorization engine that works by checking the query string of the request.
 * @package phpPoA2
 * @subpackage QueryFilterAuthorizationEngine
 */
class QueryFilterAuthzEngine extends AuthorizationEngine {

    protected $valid_hooks = array(QUERY_BEFORE_FILTERS);

    public function isAuthorized($user, $attrs) {
        $uri = $_SERVER['SERVER_NAME'].$_SERVER['REQUEST_URI'];
        $params = $this->getQueryParams();
        $allowed = $this->cfg->getAllowedPatterns();
        $denied  = $this->cfg->getDeniedPatterns();

        // run hook before checking patterns
        $args = array($uri, $params, $allowed, $denied);
        $this->runHooks(QUERY_BEFORE_FILTERS, $args);
        $uri = $args[0];
        $params = $args[1];
        $allowed = $args[2];
        $denied = $args[3];

        // check patterns against the current URI
        foreach ($allowed as $pattern) { // allowed URIs
            if (preg_match('/'.$pattern.'/', $uri)) {
                trigger_error(msg('allowed-pattern-match', array($uri, $pattern)), E_USER_WARNING);
                return true;
            }
        }

        // denied URIs
        foreach ($denied as $pattern) {
            if (preg_match('/'.$pattern.'/', $uri)) {
                trigger_error(msg('denied-pattern-match', array($uri, $pattern)), E_USER_WARNING);
                return false;
            }
        }

        // check patterns against the request params
        foreach ($params as $param) {
            // allowed params
            foreach ($allowed as $pattern) {
                if (preg_match('/'.$pattern.'/', $param)) {
                    trigger_error(msg('allowed-pattern-match', array($param, $pattern)), E_USER_WARNING);
                    return true;
                }
            }

            // denied params
            foreach ($denied as $pattern) {
                if (preg_match('/'.$pattern.'/', $param)) {
                    trigger_error(msg('denied-pattern-match', array($param, $pattern)), E_USER_WARNING);
                    return false;
                }
            }
        }

        // default response
        trigger_error(msg('authz-default-fallback'), E_USER_WARNING);
        return $this->cfg->getDefaultBehaviour();
    }

    public function getAuthorizedList() {
        $this->registerHandler();
        $list = $this->cfg->getAllowedPatterns();
        $this->clean();
        return $list;
    }

    public function authorize($user, $attrs, $ref, $expires = 0) {
        return false;
    }

    public function revoke($mail) {
        return false;
    }

    private function getQueryParams() {
        // GET queries
        $data = explode("&", $_SERVER['QUERY_STRING']);

        // POST queries
        if ($_SERVER['REQUEST_METHOD'] === "POST") {
            $str = file_get_contents(STDIN);

            $post = array();
            if ($str) {
                $post = explode("&". $str);
            }

            $data = array_merge($data, $post);
        }

        return $data;
    }

}

?>
